@@ -124,7 +124,7 @@ def dither_match(catalogues, threshold, colnames):
         base=Table(base,dtype=[float]*len(base.colnames)).filled(np.nan)
     return finish_matching(base, colnames)
 
-def generic_match(catalogues, threshold, colnames):
+def cascade_match(catalogues, threshold, colnames):
     """
     match a list of catalogues with RA and DEC columns
     INPUT: 
@@ -169,6 +169,7 @@ def generic_match(catalogues, threshold, colnames):
 
 def band_match(catalogues, threshold, colnames):
     threshold=threshold*u.arcsec
+    colnames= list( name for name in catalogues[0].colnames if name in colnames)
     ### ORDER the tables into the correct order (increasing wavelength)
     tables=np.full( len(starbug2.filters), None)
     mask=np.full(len(starbug2.filters), False)
@@ -222,12 +223,31 @@ def band_match(catalogues, threshold, colnames):
     export_table(base,"/tmp/tab.fits")
     return base
 
+def stage_match(stage2, stage3, threshold):
+    """
+    Match together a stage 2 and stage 3 catalogue
+    if the star is resolved in just the stage 3, it takes on that value
+    if the star is resolved in both, the stage3 is ignored
+    """
+    
+    if stage2.meta.get("CALIBLEVEL")!=2: perror("WARNING: stage2 catalogue CALIBLEVEL=%s does not match\n"%stage2.meta.get("CALIB_LEVEL"))
+    if stage3.meta.get("CALIBLEVEL")!=3: perror("WARNING: stage3 catalogue CALIBLEVEL=%s does not match\n"%stage2.meta.get("CALIB_LEVEL"))
+    #av,tab=dither_match((stage2,stage3),threshold,stage2.colnames)
+    idx,d2d,_=_match(stage2,stage3)
+    
+    tmp=Table(np.full((len(stage2),len(stage3.colnames)), np.nan), names=stage3.colnames)
+    for src,IDX,sep in zip(stage3,idx,d2d):
+        if (sep<threshold) and (sep==min(d2d[idx==IDX])):
+            tmp[IDX]=src
+        else: tmp.add_row(src)
 
+    mask=np.isfinite(tmp["RA"])
+    tmp.rename_columns(tmp.colnames, list("%s_stage3"%name for name in tmp.colnames))
+    tab=hstack( (stage2.copy(),tmp) )
+    tab.add_column(mask, name="MASK")
 
-
-
-
-
+    print(tab)
+    return tab 
 
 
 
@@ -259,39 +279,25 @@ def finish_matching(tab, colnames):
         
         av[name]=col
     mag,magerr=flux2ABmag(av["flux"],av["eflux"], tab.meta["FILTER"])
+    av.add_column(mag,name=tab.meta["FILTER"])
+    av.add_column(magerr,name="e%s"%tab.meta["FILTER"])
     narr= np.nansum( np.invert( np.isnan(tab2array(tab,find_colnames(tab,colnames[0])))),axis=1)
     av.add_column(Column(narr, name="NUM"))
     return (av,tab)
 
+
+def remove_NUM(tab, N):
+    """
+    Remove sources from the list of tab if they have >N non matches
+    """
+    if "NUM" in tab.colnames:
+        pass#mask=tab["NUM"]
+
+
 if __name__=="__main__":
-    #fnames=(#"/home/conor/dat/NGC346/JWST/stage2-destriped/de-striped_F335M/out.fits",
-    fnames=("/home/conor/dat/NGC346/JWST/stage2-destriped/de-striped_F200W/out.fits",
-            "/home/conor/dat/NGC346/JWST/stage2-destriped/de-striped_F444W/out.fits",
-            )
-            #"/home/conor/dat/NGC346/JWST/stage2-destriped/de-striped_F187N/out.fits")
-    tables=[Table().read(fname) for fname in fnames]
-    band_match(tables,0.5,None)
+    stage2=Table.read("/dat/1zw18/jwst/stage2/1zw18-apstage2.fits")
+    stage3=Table.read("/dat/1zw18/jwst/stage3/1Zw18-apstage3.fits")
 
-    """
-    fnames=sorted(glob.glob("/home/conor/dat/NGC346/JWST/stage2-destriped/de-striped_F335M/*-apmatch.fits"))
-    colnames=("RA","DEC","ap_flux")
-    tables=list(Table.read(fname, format="fits") for fname in fnames)
-    export_table(generic_match(tables),"out.fits")
 
-    fnames=glob.glob("/home/conor/dat/NGC346/JWST/stage2-destriped/de-striped_F187N/*-ap.fits")
-    fps=[ fits.open(fp) for fp in fnames]
-    out=sort_exposures(fps)
-
-    print("band ob visit exp det a1 a2 a3 a4 b1 b2 b3 b4 al bl ??")
-    for band,obs in out.items():
-        for ob,visits in obs.items():
-            for visit,exps in visits.items():
-                for exp,dets in exps.items():
-                    #for det,fp in dets.items():
-                    s=":     "
-                    for item in dets: s+= "X  " if item else ".. "
-
-                    ar=np.array([Table(fp[1].data._get_raw_data()) if fp else None for fp in dets])
-                    out=match_detectormodule( ar )
-                    print(band,ob,visit,exp, s)
-    """
+    tab=stage_match(stage2,stage3, 0.25*u.arcsec)
+    export_table(tab,fname="/tmp/tab.fits")
