@@ -3,10 +3,6 @@ from starbug2.utils import *
 from starbug2.misc import *
 from starbug2.routines import *
 
-import warnings
-from astropy.utils.exceptions import AstropyWarning
-warnings.simplefilter("ignore",category=AstropyWarning)
-
 from astropy.table import hstack
 
 class StarbugBase(object):
@@ -17,11 +13,14 @@ class StarbugBase(object):
     of itself from there on.
     """
     filter=None
+    stage=0
     fname=None
     detections=None
     psfcatalogue=None
     residuals=[]
     background=None
+    image=None
+    wcs=None
     def __init__(self, fname, pfile=None, options={}):
         """
         fname : FITS image file name
@@ -33,16 +32,28 @@ class StarbugBase(object):
         self.options.update(options)
         #if self.options["TEST"]: print(self.options["TEST"])
 
+        self.image=self.load_image(fname)   ## Load the fits image
         if self.options["AP_FILE"]: self.load_apfile() ## Load the source list if given
         if self.options["BGD_FILE"]: self.load_bgdfile()
-        self.image=self.load_image(fname)   ## Load the fits image
 
+    @property
+    def header(self):
+        """
+        """
+        head=fits.Header({**self.options,**self.info})
+        head["CALIBLEVEL"]=self.stage
+        head["STARBUG"]=pkg_resources.get_distribution("starbug2").version
+        head["FILTER"]=self.filter
+        return head
+
+
+    @property
     def info(self):
         """
         Get some useful information from the image header file
         """
         out={}
-        keys=("FILTER","DETECTOR")
+        keys=("FILTER","DETECTOR","TELESCOP","INSTRUME")
         if self.image:
             out.update( { (key,self.image[0].header[key]) for key in keys if key in self.image[0].header})
         return out
@@ -74,6 +85,24 @@ class StarbugBase(object):
                     if ("FILTER" in image[0].header) and (image[0].header["FILTER"] in starbug2.filters.keys()):
                         self.filter=image[0].header["FILTER"]
                     else: perror("Unable to dtermine image filter\n")
+                        
+                    ## I NEED TO DETERMINE BETTER WHAT STAGE IT IS IN
+                    exts=extnames(image)
+                    hdr=image[0].header
+
+                    if "SCI"  in exts: self.wcs=WCS(image["SCI"].header)
+                    else: self.wcs=WCS(hdr)
+
+
+                    if "DQ" in exts:
+                        if "AREA" in exts: self.stage=2
+                        else: self.stage=2.5
+                    elif "WHT" in exts: self.stage=3 
+                    else: perror("unable to determine jwst pipeline level\n")
+
+
+                    self.log("loaded: \"%s\"\n"%fname)
+                    self.log("pipeline stage: %d\n"%self.stage)
 
                 else: perror("fits file \"%s\" does not exist\n"%fname)
             else: perror("included file must be FITS format\n")
@@ -90,6 +119,14 @@ class StarbugBase(object):
             with fits.open(fname) as fp:
                 self.detections=Table(data=fp[1].data._get_raw_data())
                 self.log("loaded AP_FILE='%s'\n"%fname)
+
+                cn=self.detections.colnames
+                if not any( _ in cn for _ in ("xcentroid","ycentroid","x_0","y_0")):
+                    if all( _ in cn for _ in ("RA","DEC")):
+                        xy=self.wcs.all_world2pix(self.detections["RA"], self.detections["DEC"],0)
+                        print(xy)
+                        self.detections.add_columns(xy,names=("xcentroid","ycentroid"),indexes=[0,0])
+                    else: perror("WARNING, unable to determine physical coordinates from detections table\n")
         else: perror("AP_FILE='%s' does not exists\n"%fname)
 
     def load_bgdfile(self,fname=None):
@@ -100,7 +137,7 @@ class StarbugBase(object):
         """
         if not fname: fname=self.options["BGD_FILE"]
         if os.path.exists(fname):
-            self.background=fits.open(fname)
+            self.background=fits.open(fname)[1]
             self.log("loaded BGD_FILE='%s'\n"%fname)
         else: perror("BGD_FILE='%s' does not exists\n"%fname)
 
@@ -120,48 +157,85 @@ class StarbugBase(object):
                                         roundlo=self.options["ROUND_LO"],
                                         roundhi=self.options["ROUND_HI"],
                                         wcs=WCS(self.image[1].header),
+                                        bgd2d=self.options["DOBGD2D"],
                                         boxsize=int(self.options["BOX_SIZE"]),
                                         filtersize=int(self.options["FILTER_SIZE"]),
                                         verbose=self.options["VERBOSE"])
 
             dat=detector(self.image["SCI"].data)
-            colnames=("RA","DEC","xcentroid","ycentroid","sharpness","roundness1","roundness2","peak")
-            dat=dat[colnames]
+            colnames=("RA","DEC","xcentroid","ycentroid","sharpness","roundness1","roundness2", "peak")
+            self.detections=dat[colnames]
+            self.aperture_photometry()
 
-            self.log("Running Aperture Photometry\n")
 
-            ######################### 
-            # Unit Conversion to Jy #
-            ######################### 
+    def aperture_photometry(self):
 
-            if self.image["SCI"].header["BUNIT"]=="MJy/sr":
-                factor=1e6*float(self.image["SCI"].header["PIXAR_SR"])
-                self.image["SCI"].data*=factor
-                self.image["SCI"].header.update({"BUNIT":"Jy"})
+        if self.detections is None:
+            perror("No detection source file loaded (-d file-ap.fits)\n")
+            return
+        dat=self.detections[("RA","DEC","xcentroid","ycentroid","sharpness","roundness1","roundness2", "peak")]
+        #######################
+        # APERTURE PHOTOMETRY #
+        #######################
+        self.log("Running Aperture Photometry\n")
+        image=self.image["SCI"].data.copy() ##dont work on the real image!
 
-                if "ERR" in extnames(self.image) and np.shape(self.image["ERR"]):
-                    self.image["ERR"].data*=factor
-                    self.image["ERR"].header.update({"BUNIT":"Jy"})
+        ######################### 
+        # Unit Conversion to Jy #
+        ######################### 
+        error=None
+        scalefactor=1
+        if self.image["SCI"].header["BUNIT"]=="MJy/sr":
+            scalefactor=1e6*float(self.image["SCI"].header["PIXAR_SR"])
+            self.log("-> converting unit from MJy/sr to Jr with factor: %e\n"%scalefactor)
 
-                self.log("-> converting unit from MJy/sr to Jr with factor: %e\n"%factor)
+        image*=scalefactor
+        if "ERR" in extnames(self.image) and np.shape(self.image["ERR"]):
+            error=self.image["ERR"].data
+            error*=scalefactor
+        else: error=np.sqrt(image)
 
-            apphot=APPhot_Routine( self.options["APPHOT_R"], self.options["SKY_RIN"], self.options["SKY_ROUT"], verbose=self.options["VERBOSE"])
-            apcorr=apphot.calc_apcorr( self.filter, self.options["APPHOT_R"],"%s/jwst_nircam_apcorr_0004.fits"%(self.options["PSFDIR"] ))
-            ap_cat=apphot(dat, self.image, apcorr=apcorr, sig_sky=self.options["SIGSKY"])
 
-            mag,magerr=flux2ABmag( ap_cat["flux"], ap_cat["eflux"], filter=self.filter)
-            ap_cat.add_column(Column(mag,self.filter))
-            ap_cat.add_column(Column(magerr,"e%s"%self.filter))
 
-            #ap_cat.add_column(Column( -2.5*np.log10(ap_cat["flux"] / ZP[self.filter][0]), name="%s_mag"%self.filter))
-            #ap_cat.add_column(Column( 2.5*np.log10(1+(ap_cat["eflux"]/ap_cat["flux"])), name="%s_emag"%self.filter ))
+        #######################
+        # Aperture Correction #
+        #######################
+        apcorr=1
+        fname=None
+        radius=self.options["APPHOT_R"]
+        skyin= self.options["SKY_RIN"]
+        skyout=self.options["SKY_ROUT"]
 
-            self.detections=hstack((dat,ap_cat))
-            self.detections.meta=self.info()
-            self.detections.meta.update({"ROUNTINE":"DETECT"})
+        if   self.info["INSTRUME"]=="NIRCAM": fname="%s/apcorr_nircam.fits"%self.options["PSFDIR"]
+        elif self.info["INSTRUME"]=="MIRI":   fname="%s/apcorr_miri.fits"%self.options["PSFDIR"]
+        else: perror("No apcorr file available for instrument\n")
 
-        else: perror("Failed to run aperture photometry")
+        if self.options["FIT_APP_R"]:
+            apcorr=APPhot_Routine.calc_apcorr(self.filter, self.options["APPHOT_R"], table_fname=fname, verbose=self.options["VERBOSE"])
+        else:
+            apcorr,radius=APPhot_Routine.apcorr_from_encenergy(self.filter,self.options["ENCENERGY"],table_fname=fname, verbose=self.options["VERBOSE"])
 
+        apphot=APPhot_Routine( radius, skyin, skyout, encircled_energy=self.options["ENCENERGY"], fit_radius=self.options["FIT_APP_R"], verbose=self.options["VERBOSE"])
+
+        if self.stage==2:
+            image*= self.image["AREA"].data ## AREA distortion correction
+            mask=self.image["DQ"].data & (DQ_DO_NOT_USE|DQ_SATURATED) #|DQ_JUMP_DET)
+            image[mask]=np.nan
+            error[mask]=np.nan
+            ap_cat=apphot(dat, image, error=error, dqflags=self.image["DQ"].data, apcorr=apcorr, sig_sky=self.options["SIGSKY"])
+
+        else: ##stage 3 version
+            ap_cat=apphot(dat, image, error=error, apcorr=apcorr, sig_sky=self.options["SIGSKY"])
+
+        mag,magerr=flux2ABmag( ap_cat["flux"], ap_cat["eflux"], filter=self.filter)
+        ap_cat.add_column(Column(mag,self.filter))
+        ap_cat.add_column(Column(magerr,"e%s"%self.filter))
+
+        self.detections=hstack((dat,ap_cat))
+        self.detections.meta=dict(self.header.items())
+        self.detections.meta.update({"ROUNTINE":"DETECT"})
+
+        #else: perror("Failed to run aperture photometry")
 
     def bgd_estimate(self):
         """
@@ -170,7 +244,15 @@ class StarbugBase(object):
         """
         self.log("Estimating Background\n")
         if self.detections:
-            bgd=BackGround_Estimate_Routine(self.detections, 
+            xname="xcentroid" if "xcentroid" in self.detections.colnames else "x_0"
+            yname="ycentroid" if "ycentroid" in self.detections.colnames else "y_0"
+
+            sources=self.detections[[xname,yname]]
+            sources=sources[ sources[xname]>=0 ]
+            sources=sources[ sources[yname]>=0 ]
+            sources=sources[ sources[xname]<self.image["SCI"].header["NAXIS1"]]
+            sources=sources[ sources[yname]<self.image["SCI"].header["NAXIS2"]]
+            bgd=BackGround_Estimate_Routine(sources, 
                                             boxsize=int(self.options["BOX_SIZE"]),
                                             fwhm=starbug2.filters[self.filter][2],
                                             verbose=self.options["VERBOSE"])
@@ -178,6 +260,16 @@ class StarbugBase(object):
         else:
             perror("unable to estimate background, no source list loaded\n")
 
+    def bgd_subtraction(self):
+        
+        self.log("Subtracting Background\n")
+
+        if self.background is None:
+            perror("No background array loaded (-b file-bgd.fits)\n")
+            return 
+        array= self.image["SCI"].data - self.background.data
+        self.residuals.append(array)
+        self.image["SCI"].data=array
 
     def photometry(self):
         """
@@ -185,11 +277,11 @@ class StarbugBase(object):
         Saves the result as a table self.psfcatalogue
         Additionally it appends a residual Image onto the self.residuals HDUList
         """
-        if not self.detections:
+        if self.detections is None:
             perror("unable to run photometry: no source list loaded\n")
             return
 
-        if not self.background:
+        if self.background is None:
             perror("unable to run photometry: no background estimation loaded\n")
             return
 
@@ -200,6 +292,9 @@ class StarbugBase(object):
             with fits.open(fname) as fp:
                 psf_model=DiscretePRF(fp[0].data)
 
+                print(psf_model.fixed)
+                psf_model.fixed["x_0"]=False
+                psf_model.fixed["y_0"]=False
             phot=PSFPhot_Routine(   self.options["CRIT_SEP"],
                                     starbug2.filters[self.filter][2],
                                     psf_model,
@@ -210,20 +305,28 @@ class StarbugBase(object):
                                     sharphi=self.options["SHARP_HI"],
                                     roundlo=self.options["ROUND_LO"],
                                     roundhi=self.options["ROUND_HI"],
-                                    #boxsize=self.options["BOX_SIZE"],
-                                    background=self.background[1].data,
-                                    #filtersize=self.options["FILTER_SIZE"],
-                                    wcs=WCS(self.image[1].header))
+                                    background=self.background.data,
+                                    wcs=self.wcs)
 
             init_guesses=self.detections.copy()
+
             if "xcentroid" in init_guesses.colnames: init_guesses.rename_column("xcentroid", "x_0")
             if "ycentroid" in init_guesses.colnames: init_guesses.rename_column("ycentroid", "y_0")
 
-            init_guesses.remove_columns(("sharpness", "roundness1", "roundness2", "npix", "sky", "flux", "mag", "ap_flux", "ap_sky"))
+            init_guesses=init_guesses[ init_guesses["x_0"]>=0 ]
+            init_guesses=init_guesses[ init_guesses["y_0"]>=0 ]
+            init_guesses=init_guesses[ init_guesses["x_0"]<self.image["SCI"].header["NAXIS1"]]
+            init_guesses=init_guesses[ init_guesses["y_0"]<self.image["SCI"].header["NAXIS2"]]
 
+
+            init_guesses=init_guesses[["x_0","y_0"]]
+
+            with open("out.reg","w") as fp:
+                for line in init_guesses:
+                    fp.write("circle %f %f 3\n"%( line["x_0"]+1, line["y_0"]+1))
             self.psfcatalogue=tabppend(self.psfcatalogue, phot(self.image[1].data, init_guesses=init_guesses))
             self.residuals.append(phot.get_residual_image())
-            self.background=fits.ImageHDU(data=phot.bkg_estimator.bgd, name="BACKGROUND")
+            self.background=fits.ImageHDU(data=phot.bkg_estimator.bgd, name="BACKGROUND") ##So is it supposed to be a fits image or a numpy array?!
 
     def cleanup(self):
         """
@@ -294,14 +397,15 @@ class StarbugBase(object):
             perror("output directory '%s' does not exist, using /tmp instead\n"%outdir)
             outdir="/tmp"
 
-        head=fits.Header(self.options)
         dname,fname,ext=split_fname(self.fname)
         if self.detections: 
-            hdulist=[fits.PrimaryHDU(header=head),fits.BinTableHDU(data=self.detections)]
+            reindex(self.detections)
+            hdulist=[fits.PrimaryHDU(header=self.header),fits.BinTableHDU(data=self.detections)]
             fits.HDUList(hdulist).writeto("%s/%s-ap.fits"%(outdir,fname), overwrite=True)
             #export_table(self.detections, fname="%s/%s-ap.fits"%(outdir,fname))
         if self.psfcatalogue: 
-            hdulist=[fits.PrimaryHDU(header=head),fits.BinTableHDU(data=self.psfcatalogue)]
+            reindex(self.psfcatalogue)
+            hdulist=[fits.PrimaryHDU(header=self.header),fits.BinTableHDU(data=self.psfcatalogue)]
             fits.HDUList(hdulist).writeto("%s/%s-psf.fits"%(outdir,fname), overwrite=True)
             #export_table(self.psfcatalogue, fname="%s/%s-psf.fits"%(outdir,fname))
         
@@ -329,23 +433,34 @@ class StarbugBase(object):
             1 - on fail
         """
         status=0
+        warn=lambda :perror(sbold("WARNING: "))
 
         if self.filter not in starbug2.filters.keys():
-            perror("WARNING: Unknown filter '%s'\n"%self.filter)
+            warn()
+            perror("Unknown filter '%s'\n"%self.filter)
             status=1
 
         dname = os.path.expandvars(self.options["PSFDIR"])
         if not os.path.exists(dname):
-            perror("WARNING: Unable to locate PSFDIR='%s'\n"%dname)
+            warn()
+            perror("Unable to locate PSFDIR='%s'\n"%dname)
             status=1
 
         else:
             if not os.path.exists("%s/%s.fits"%(dname, self.filter)):
-                perror("WARNING: Unable to locate filter PSF for '%s'\n"%self.filter)
+                warn()
+                perror("Unable to locate filter PSF for '%s'\n"%self.filter)
                 status=1
         
         if not os.path.exists((dname:=os.path.expandvars(self.options["OUTDIR"]))):
-            perror("WARNING: Unable to locate OUTDIR='%s'\n"%dname)
+            warn()
+            perror("Unable to locate OUTDIR='%s'\n"%dname)
+            status=1
+
+        tmp=load_params("%sdefault.param"%pkg_resources.resource_filename("starbug2","param/"))
+        if set(tmp.keys()) - set(self.options.keys()):
+            warn()
+            perror("parameter file version mismatch. Run starbug --local-param to update\n")
             status=1
 
         return status
@@ -358,4 +473,7 @@ class StarbugBase(object):
 
     def __setstate__(self, state):
         self.__dict__.update(state)
+        v=self.options["VERBOSE"]
+        self.options["VERBOSE"]=0
         self.load_image(self.fname)
+        self.options["VERBOSE"]=v
